/*
 * This file is part of Jstacs.
 *
 * Jstacs is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Jstacs is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Jstacs.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * For more information on Jstacs, visit http://www.jstacs.de
 */

package de.jstacs.parameters.validation;

import de.jstacs.NonParsableException;
import de.jstacs.Storable;
import de.jstacs.classifier.AbstractClassifier;
import de.jstacs.io.XMLParser;
import de.jstacs.models.AbstractModel;
import de.jstacs.models.Model;
import de.jstacs.parameters.FileParameter;

/**
 * Class for a validator that validates instances and XML-representations for the correct class types (e.g. <code>AbstractModel</code>
 * @author Jan Grau
 *
 */
public class StorableValidator implements ParameterValidator {
    
    /**
     * The error message, <code>null</code> if no error occurred
     */
    private String errorMessage;
    /**
     * the class of the object
     */
    private Class clazz;
    /**
     * <code>true</code> if the model/classifier must be trained
     */
    private boolean trained;
    
    /**
     * Creates a new <code>ObjectValidator</code> for a subclass of <code>AbstractModel</code>
     * or <code>AbstractClassifier</code>. This constructor may not be used on other subclasses of <code>Storable</code>
     * @param clazz the class
     * @param trained true if the model or classifier must be trained
     * @throws Exception an exception is thrown if <code>clazz</code> is not of expected type
     */
    public StorableValidator(Class<? extends Storable> clazz, boolean trained) throws Exception{
        if(AbstractModel.class.isAssignableFrom(clazz) ||
                AbstractClassifier.class.isAssignableFrom(clazz)){
            this.clazz = clazz;
        }else{
            throw new Exception("Class is not a subtype of StorableAbstractClassifier or StorableAbstractModel");
        }
        this.trained = trained;
    }
    
    /**
     * Creates a new <code>ObjectValidator</code> for a subclass of <code>Storable</code>.
     * @param clazz the class
     * @throws Exception an exception is thrown if <code>clazz</code> is not of expected type
     */
    public StorableValidator(Class<? extends Storable> clazz) throws Exception{
        if(Storable.class.isAssignableFrom(clazz)){
            this.clazz = clazz;
        }else{
            throw new Exception("Class is not a subtype of Storable");
        }
        this.trained = false;
    }
    
    /**
     * Constructs an <code>ObjectValidator</code> from its XML-representation
     * @param buf the XML-representation
     * @throws NonParsableException a <code>NonParsableException</code> is thrown if <code>buf</code> could not be parsed
     */
    public StorableValidator(StringBuffer buf) throws NonParsableException{
        fromXML(buf);
    }
    
    
    public StorableValidator clone() throws CloneNotSupportedException{
        try{
            StorableValidator clone = null;      
            if(AbstractModel.class.isAssignableFrom(clazz) ||
                    AbstractClassifier.class.isAssignableFrom(clazz)){
                clone = new StorableValidator((Class<? extends Storable>)clazz,trained);
            }else{
                clone = new StorableValidator((Class<? extends Storable>)clazz);
            }
            clone.errorMessage = errorMessage;
            return clone;
        }catch(Exception e){
            throw new CloneNotSupportedException(e.getCause().getMessage());
        }
    }
    
    /**
     * Checks the value of <code>value</code>. Allowed types of <code>value</code> are <code>AbstractModel</code>,
     * <code>AbstractClassifier</code>, <code>FileRepresentation</code>, <code>String</code>, and <code>StringBuffer</code>.
     * In all cases where an XML-representation is given as <code>value</code>, it must be surrounded by &lt;object&gt;-tags and
     * these tags must contain a &lt;className&gt;-element that contains the name of the class of the represented instance.
     * @param value the value to be checked.
     */
    @SuppressWarnings("unchecked")
    public boolean checkValue(Object value) {
        if(AbstractModel.class.isAssignableFrom(clazz) && value instanceof AbstractModel){
            if(!trained || ( (Model) value).isTrained()){
                errorMessage = null;
                return true;
            }else{
                errorMessage = "The model must be trained.";
                return false;
            }
        }else if(AbstractClassifier.class.isAssignableFrom(clazz) && value instanceof AbstractClassifier){
            if(!trained || ( (AbstractClassifier) value).isTrained()){
                errorMessage = null;
                return true;
            }else{
                errorMessage = "The classifier must be trained.";
                return false;
            }
        }else{
            
            StringBuffer buf = null;
            if(value instanceof FileParameter.FileRepresentation){
                String content = ( (FileParameter.FileRepresentation) value).getContent();
                buf = new StringBuffer(content);
            }else if(value instanceof String){
                buf = new StringBuffer( (String) value );
            }else if(value instanceof StringBuffer){
                buf = (StringBuffer) value;
            }
            
            try{
                buf = XMLParser.extractForTag(buf,"object");
                String className = XMLParser.extractStringForTag(buf,"className");
                Class c = Class.forName(className);
                if(clazz.isAssignableFrom(c)){
                    boolean modelTrained = false;
                    if(AbstractModel.class.isAssignableFrom(c)){
                        Model model = (Model) c.getConstructor(new Class[]{StringBuffer.class}).newInstance(buf);
                        modelTrained = model.isTrained();
                    }else if(AbstractClassifier.class.isAssignableFrom(c)){
                        AbstractClassifier classifier = (AbstractClassifier) c.getConstructor(new Class[]{StringBuffer.class}).newInstance(buf);
                        modelTrained = classifier.isTrained();
                    }else{
                        c.getConstructor(new Class[]{StringBuffer.class}).newInstance(buf);
                    }
                    if(!trained || modelTrained){
                        errorMessage = null;
                        return true;
                    }else{
                        errorMessage = "The classifier must be trained.";
                        return false;
                    }
                }else{
                    errorMessage = "File content was not of the correct class "+clazz.getName()+" but of "+className;
                    return false;
                }
            }catch(Exception e){
                e.printStackTrace();
                errorMessage = "The file could not be parsed: "+e.getCause();
                return false;
            }
        }
    }
    
    public String getErrorMessage() {
        return errorMessage;
    }
    
    public StringBuffer toXML() {
        StringBuffer buf = new StringBuffer();
        XMLParser.appendStringWithTags(buf,errorMessage,"errorMessage");
        XMLParser.appendBooleanWithTags(buf,trained,"trained");
        XMLParser.appendStringWithTags(buf,clazz.getName(),"class");
        XMLParser.addTags(buf,"TrainedValidator");
        
        return buf;
    }
    
    /**
	 * Parses a {@link StorableValidator} from the XML-representation
	 * as returned by {@link StorableValidator#toXML()}.
	 * @param representation the XML-representation
	 * @throws NonParsableException thrown if the XML-code could not be parsed
	 */
    public void fromXML(StringBuffer representation) throws NonParsableException {
        representation = XMLParser.extractForTag(representation,"TrainedValidator");
        errorMessage = XMLParser.extractStringForTag(representation,"errorMessage");
        trained = XMLParser.extractBooleanForTag(representation,"trained");
        try{
            clazz = Class.forName( XMLParser.extractStringForTag(representation,"class") );
        }catch(Exception e){
            throw new NonParsableException(e.getMessage());
        }
    }
    
    
    
}
